<?php
/*
Plugin Name: LDAP WP Login Integration With Active Directory.
Plugin URI: https://www.ldapsso.com
Description: Login and authenticate Wordpress users using AD credentials
Version: 1.0.0
Author: robert05 
Author URI: https://www.ldapsso.com
*/

require_once 'ldap_auth_layout.php';
require_once 'ldap_AD_config.php';
class lwliad_Ladp_auth{

    protected static $instance = NULL;

    public static function getInstance () {
		if ( !self::$instance ) {
		  self::$instance = new self;
		}
		return self::$instance;
    }
    
    function __construct(){
        add_action('admin_menu',array($this,'addMenuPage'));
        add_action('init',array($this,'save_ldap_config'));
        // add_action('admin_notices', 'messages');
        if(get_option('lwliad_AD_ldapLogin') == 1){
          add_filter('authenticate',array($this,'authenticateUser'), 7, 3); 
        }
      
        

    }

    function addMenuPage(){
        add_menu_page('LDAP_auth_layout','LDAP','manage_options','LDAP authentication intergrating with AD', 'lwliad_ladp_auth_layout');
    }


    /* Function runs on user login */


    function authenticateUser($user, $username, $password){
      if(empty($username) || empty($password)){

         $error = new WP_Error();

        if(empty($username)){
          $error->add('empty_username', __('<strong>ERROR</strong>: Email field is empty.'));
        }
        if(empty($password)){
          $error->add('empty_password', __('<strong>ERROR</strong>: Password field is empty.'));
        }

        return $error;


      }
      elseif(!empty($username)){
     
        $result =lwliad_AD_Config::lwliad_authenticate($username,$password);
     
      
        if($result){   
          

          /**Fetching attributes and mapping to WP fields */
  

          $attributesOfAD =lwliad_AD_Config::fetch_attributes_for_username($username);
         
          $user_id = Self::check_WP_user($attributesOfAD[0][strtolower(get_option( 'lwliad_AD_attr_map_email' ))][0]);
          

        if(!empty(get_option( 'lwliad_AD_attr_map_email' ))){    
           

          $user_info = array();
          $user_info['first_name'] = $attributesOfAD[0][strtolower(get_option( 'lwliad_AD_attr_map_firstName'))][0];
          $user_info['last_name'] = $attributesOfAD[0][strtolower(get_option( 'lwliad_AD_attr_map_lastName' ))][0];
          $user_info['user_email'] = $attributesOfAD[0][strtolower(get_option( 'lwliad_AD_attr_map_email' ))][0];
          $user_info['user_pass'] = $password;
          // $user_info['user_url'] = $attribs[0][strtolower($authLDAPWebAttr)][0];
          
            $user_info['display_name'] = $attributesOfAD[0][strtolower(get_option('lwliad_AD_attr_map_email'))][0];
            $user_info['nickname'] = $attributesOfAD[0][strtolower(get_option( 'lwliad_AD_attr_map_email' ))][0];
           
            $user_info['user_login'] = $attributesOfAD[0][strtolower(get_option( 'lwliad_AD_Config_filter' ))][0];

         }else{

            if($attributesOfAD[0]['userprincipalname'][0]){

                  $ad_email_set = $attributesOfAD[0]['userprincipalname'][0];

              }else{

                  $ad_email_set = $attributesOfAD[0]['mail'][0];

              }


                $user_info = array();
                $user_info['first_name'] = $ad_email_set;
                $user_info['user_email'] = $ad_email_set;
                $user_info['user_pass'] = $password;
                $user_info['user_login'] = $username;


        }

    

  
       
        
          if($user_id){
           
            $user = wp_update_user($user_info);
          }else{
            
            $user =wp_insert_user($user_info);
           
          }
          /** Role Mapping */
          if(get_option('lwliad_enabledefaultRole')){

             
                lwliad_AD_Config::set_users_role($user,$username);
           }          
      
          return $user;
         

        }
        else if(get_option('lwliad_AD_ldapLogin') == 1 && get_option('lwliad_disableWPLogin') > 0){
          remove_filter('authenticate', 'wp_authenticate_username_password', 20, 3);
          remove_filter('authenticate', 'wp_authenticate_email_password', 20, 3);

        }
      

       
      }
    }

    /** Check for user exist in WP */

    function check_WP_user($username){

      
    

      global $wpdb;

      $user_id = $wpdb->get_var($wpdb->prepare("SELECT * FROM $wpdb->users WHERE user_email= %s", $username  ));


      if($user_id){
        return $user_id;

      }else{
        return null;
      }
      
    }

    /**  */


    function save_ldap_config(){
      
      if(isset($_POST['action'])){
        if ($_POST['action'] == 'ldapConfig')
      {
   
    
        if(isset($_POST['ldapConfig_nonce']) && !empty( $_POST['ldapConfig_nonce'])){
   
            update_option('lwliad_AD_Config_ldapURl', isset($_POST['ldapURI'])?sanitize_text_field($_POST['ldapURI']):'');         

            update_option('lwliad_AD_Config_ldapport',isset($_POST['ldapport'])?sanitize_text_field($_POST['ldapport']):'');
                   
            update_option('lwliad_AD_Config_ldapDn', isset($_POST['ldapDN'])?sanitize_text_field($_POST['ldapDN']):'');
          
    
            update_option('lwliad_AD_Config_ldappassword', isset($_POST['ldappassword'])?$_POST['ldappassword']:'');
          

          if(isset($_POST['ldapFilter']) && !empty($_POST['ldapFilter']) ){
            $filter = "(&(objectCategory=*)(".sanitize_text_field($_POST['ldapFilter'])."=?))";
            
            update_option('lwliad_AD_Config_ldapFilter', $filter);
            update_option('lwliad_AD_Config_filter', sanitize_text_field($_POST['ldapFilter']));
          }

          $result =lwliad_AD_Config::lwliad_AD_setup();
          if($result){

            if(isset($result["Error"])){
              self::error($result["Error"]);
            }else{
              self::success('AD bind Successfully !!!!!!!!!');
            }
            
            
          }

        }
    
      
      }elseif($_POST['action'] == 'testLdapConfig'){
       
        if( isset($_POST['testldapConfig_nonce']) && !empty( $_POST['testldapConfig_nonce'])){
          
             
                if(empty($_POST['username']) || empty($_POST['password'])){

                    if(empty($_POST['username'])){
                        self::error('Please enter username.');
                    }
                    if(empty($_POST['password'])){
                        self::error('Please enter password.');
                    }
                }else{

                  
                    $result = lwliad_AD_Config::testConfig(sanitize_text_field($_POST['username']),$_POST['password']);

                    if(isset($result["Error"])){
                      self::error($result["Error"]);
                    }else{
                      self::success($result["Success"]);
                    }

                }
        }            

      }elseif($_POST['action'] == 'ldapsettings'){
        if(isset($_POST['ldapsettings_nonce']) && !empty( $_POST['ldapsettings_nonce'])){
        
       
            update_option('lwliad_AD_ldapLogin', isset($_POST['ldapLogin']) ? 1 : 0);
          
            update_option('lwliad_disableWPLogin', isset($_POST['disableWPLogin'])? 1:0);
          
         

        }

      }elseif($_POST['action'] == 'testattributeMappingSettings'){
        if(isset($_POST['testattributeMappingSettings_nonce']) && !empty( $_POST['testattributeMappingSettings_nonce'])){
          if(isset($_POST['ldapUsername'])){
            $result = lwliad_AD_Config::fetch_attributes_for_username(sanitize_text_field($_POST['ldapUsername']));
          }else{
            self::error('Please enter username.');
          }
         

                
        }

      }elseif($_POST['action'] == 'attributeMappingSettings'){
        if(isset($_POST['attributeMappingSettings_nonce']) && !empty( $_POST['attributeMappingSettings_nonce'])){

          update_option('lwliad_AD_attr_map_email', isset($_POST['ldapemail'])?sanitize_text_field($_POST['ldapemail']):'');
                
          update_option('lwliad_AD_attr_map_firstName', !empty($_POST['ldapFirstName'])? sanitize_text_field($_POST['ldapFirstName']):'');
          
          update_option('lwliad_AD_attr_map_lastName', !empty($_POST['ldapLastName'])?sanitize_text_field($_POST['ldapLastName']):'');
          
        }

      }elseif($_POST['action'] == 'groupMappingSettings'){
        if(isset($_POST['groupMappingSettings_nonce']) && !empty( $_POST['groupMappingSettings_nonce'])){

          
            update_option('lwliad_deaultRole',!empty($_POST['deaultRole'])?sanitize_text_field($_POST['deaultRole']):'');
          

            update_option('lwliad_enabledefaultRole', isset($_POST['enabledefaultRole']) ? 1 :0 );
          

            update_option('lwliad_AD_roleAgainstAdmin', !empty($_POST['roleAgainstAdmin'])? sanitize_text_field($_POST['roleAgainstAdmin']): '');
          
         
            update_option('lwliad_AD_roleAgainstEditor', !empty($_POST['roleAgainstEditor']) ? sanitize_text_field($_POST['roleAgainstEditor']): '');
          


            update_option('lwliad_AD_roleAgainstSubscriber', !empty($_POST['roleAgainstSubscriber']) ? sanitize_text_field($_POST['roleAgainstSubscriber']): '');
       
            update_option('lwliad_AD_roleAgainstContributer', !empty($_POST['roleAgainstContributer']) ? sanitize_text_field($_POST['roleAgainstContributer']):'');
          
         

        }

      } 
      }
   
      
      

     
        
    }

    /* Notifications on success and error messages */
    
    function success($message){
      $class = 'notice notice-success is-dismissible';
      printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) ); 
    }
    function error($message){
      $class = 'notice notice-error';
      printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) ); 
    }





}
$Ldap_auth = lwliad_Ladp_auth::getInstance();